class JustNimbusError(Exception):
    pass


class InvalidClientID(JustNimbusError):
    def __init__(self, client_id: str):
        super().__init__(f"{client_id!r} is an invalid client_id")
        self.client_id = client_id



import dataclasses
from datetime import datetime


@dataclasses.dataclass
class JustNimbusModel:
    api_version: str
    system_last_update: datetime
    system_flow: str
    reservoir_capacity: float
    reservoir_content: float
    reservoir_temp: float
    pump_type: str
    pump_pressure: float
    water_saved: float
    water_used: float

    @classmethod
    def from_dict(cls, data: dict):
        return JustNimbusModel(
            api_version=str(data["api_version"]),
            system_last_update=datetime.fromisoformat(data["system_last_update"]),
            system_flow=str(data["system_flow"]),
            reservoir_capacity=_to_float(data["reservoir_capacity"]),
            reservoir_content=_to_float(data["reservoir_content"]),
            reservoir_temp=_to_float(data["reservoir_temp"]),
            pump_type=str(data["pump_type"]),
            pump_pressure=_to_float(data["pump_pressure"]),
            water_saved=_to_float(data["water_saved"]),
            water_used=_to_float(data["water_used"]),
        )


def _to_float(value: str) -> float | None:
    try:
        return float(value)
    except ValueError:
        return None



import requests

from justnimbus.exceptions import InvalidClientID, JustNimbusError
from justnimbus.model import JustNimbusModel


class JustNimbusClient:
    def __init__(self, client_id: str, zip_code: str):
        self._client_id = client_id
        self._zip_code = zip_code

    def get_data(self) -> JustNimbusModel:
        url = f"https://dashboard.justnimbus.com/user/view.php?system={self._client_id}&zip={self._zip_code}&output=json"
        response = requests.get(url=url)

        try:
            response.raise_for_status()
            return JustNimbusModel.from_dict(response.json()[0])
        except requests.HTTPError as error:
            if response.status_code == 404:
                raise InvalidClientID(client_id=self._client_id) from error
            raise JustNimbusError() from error
        except LookupError as error:
            raise JustNimbusError() from error
